﻿using C1.Win.C1TrueDBGrid;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace TrueDBGrid_checkboxes
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
            c1TrueDBGrid1.DataSource = GetData();
            c1TrueDBGrid1.OwnerDrawCell += C1TrueDBGrid1_OwnerDrawCell;
            c1TrueDBGrid1.Splits[0].DisplayColumns[0].OwnerDraw = true;
            c1TrueDBGrid1.BeforeColEdit += C1TrueDBGrid1_BeforeColEdit;
            c1TrueDBGrid1.MouseClick += C1TrueDBGrid1_MouseClick;            
            c1TrueDBGrid1.RowHeight = 34;            
        }



        private void C1TrueDBGrid1_BeforeColEdit(object sender, BeforeColEditEventArgs e)
        {
            // is custom checkbox? - cancel edit, grid checks default checkbox and can change value after MouseClick
            e.Cancel = e.Column.OwnerDraw;            
        }

        private void C1TrueDBGrid1_MouseClick(object sender, MouseEventArgs e)
        {
            if (c1TrueDBGrid1.CellContaining(e.X,e.Y, out int row, out int col))
            {
                // is custom checkbox?
                if (c1TrueDBGrid1.Splits[0].DisplayColumns[col].OwnerDraw)
                {
                    var cell = c1TrueDBGrid1.Splits[0].GetCellBounds(row, col);
                    var style = c1TrueDBGrid1.Splits[0].GetCellStyle(row, c1TrueDBGrid1.Splits[0].DisplayColumns[col]);

                    Rectangle imageRect = new Rectangle(cell.X, cell.Y, ilCheckBoxes.ImageSize.Width, ilCheckBoxes.ImageSize.Height);
                    CenterRectV(ref imageRect, cell, style.VerticalAlignment);
                    CenterRectH(ref imageRect, cell, style.HorizontalAlignment);
                    if (imageRect.Contains(e.Location))
                    {
                        c1TrueDBGrid1.Col = col;
                        c1TrueDBGrid1.Row = row;
                        bool.TryParse(c1TrueDBGrid1[row, col]?.ToString(), out bool isChecked);
                        c1TrueDBGrid1[row, col] = !isChecked;
                        c1TrueDBGrid1.Invalidate();
                    }
                }
            }
        }

        private void C1TrueDBGrid1_OwnerDrawCell(object sender, OwnerDrawCellEventArgs e)
        {
            if (e.Column.DataColumn.DataType == typeof(bool))
            {                
                var oldClip = e.Graphics.ClipBounds;
                e.Graphics.SetClip(e.CellRect);
                // background
                using (var brush = new SolidBrush(e.Style.BackColor))
                    e.Graphics.FillRectangle(brush, e.CellRect);
                // checkbox                
                var point = e.CellRect.Location;
                point.X += (e.CellRect.Width - ilCheckBoxes.ImageSize.Width) / 2;
                bool.TryParse(c1TrueDBGrid1[e.Row, e.Col]?.ToString(), out bool isChecked);
                if (isChecked)
                {
                    e.Graphics.DrawImage(ilCheckBoxes.Images[0], point);
                }
                else
                {
                    e.Graphics.DrawImage(ilCheckBoxes.Images[1], point);
                }
                e.Graphics.SetClip(oldClip);
                e.Handled = true;
            }
        }

        private void CenterRectV(ref Rectangle Source, Rectangle Measure, AlignVertEnum Alignment)
        {
            switch (Alignment)
            {
                case AlignVertEnum.Top:
                    Source.Y = Measure.Top;
                    break;
                case AlignVertEnum.Center:
                    Source.Y = Measure.Top + (Measure.Height - Source.Height) / 2 + 1;
                    break;
                case AlignVertEnum.Bottom:
                    Source.Y = Measure.Bottom - Source.Height;
                    break;
            }
        }

        private void CenterRectH(ref Rectangle Source, Rectangle Measure, AlignHorzEnum Alignment)
        {
            switch (Alignment)
            {
                case AlignHorzEnum.Near:
                    Source.X = Measure.Left;
                    break;
                case AlignHorzEnum.Far:
                    Source.X = Measure.Right - Source.Width;
                    break;
                case AlignHorzEnum.Center:
                case AlignHorzEnum.General:
                    Source.X = Measure.Left + (Measure.Width - Source.Width) / 2;
                    break;
            }
        }

        private BindingSource GetData()
        {
            var table = new DataTable();
            table.Columns.Add("Custom CheckBox bool", typeof(bool));
            table.Columns.Add("CheckBox bool", typeof(bool));
            Enumerable.Range(0, 10).Select(x => new object[] { x % 2 == 0, x % 2 == 0 }).ToList().ForEach(x =>
                table.Rows.Add(x)
                );
            return new BindingSource()
            {
                DataSource = table
            };
        }
    }
}
